﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using AdrianKousz.Util;

namespace AdrianKousz.GenieEngine
{
	public static class BitmapHelpers
	{
		public static unsafe void FillMap(Scenario scn, Bitmap bmp, Bitmap bmp2)
		{
			using (var ubmp = new UnsafeBitmap(bmp))
			using (var ubmp2 = new UnsafeBitmap(bmp2))
			{
				var factory = new ScnDefaultFactory();

				Check(ubmp);
				Check(ubmp2);

				var ptr = ubmp.Scan0;
				var last = ptr + ubmp.Stride * ubmp.Height;

				var ptr2 = ubmp2.Scan0;
				var last2 = ptr2 + ubmp2.Stride * ubmp2.Height;

				var map = factory.MakeMap();
				map.SizeX = ubmp.Width;
				map.SizeY = ubmp.Height;
				map.LinearTiles = new Scenario.ScnMap.Tile[map.SizeX * map.SizeY];

				var gaiaunits = new List<Scenario.ScnUnit>();

				var treemapping = new short[256];
				treemapping[10] = 411;
				treemapping[13] = 351;
				treemapping[17] = 414;
				treemapping[18] = 348;
				treemapping[19] = 350;
				treemapping[20] = 349;

				var i = 0;
				while (ptr < last && ptr2 < last2) {
					var treeid = treemapping[*ptr];
					if (treeid != 0) {
						var unit = factory.MakeUnit();
						var posy = i / map.SizeX;
						var posx = i - (posy * map.SizeX);
						unit.UnitId = treeid;
						unit.PosX = posx + 0.5f;
						unit.PosY = posy + 0.5f;
						unit.Rotation = (short)Util.Math.Rand(0, 13);
						gaiaunits.Add(unit);
					}
					map.LinearTiles[i++] = new Scenario.ScnMap.Tile(*ptr++, (byte)(*ptr2++ / 37), 0);
				}

				scn.Map = map;
				scn.Units[0] = gaiaunits;
			}
		}

		private static void Check(UnsafeBitmap ubmp)
		{
			if (ubmp.PixelFormat != PixelFormat.Format8bppIndexed)
				throw new ArgumentException("Unsupported PixelFormat");
			if (ubmp.Stride != ubmp.Width)
				throw new NotImplementedException("bmpbit.Stride != bmpbit.Width");
		}
	}
}

