﻿using System.IO;
using System.Drawing;
using AdrianKousz.Util;

namespace AdrianKousz.GenieEngine
{
	public class Bmp2MapCommand : CmdApp.Command
	{
		override public void Run(string[] args)
		{
			using (var srcstream = File.OpenRead(args[0]))
			using (var outstream = File.OpenWrite(args[1]))
			using (var bmpstream = File.OpenRead(args[2]))
			using (var bmp2stream = File.OpenRead(args[3]))
			{
				var bmp = (Bitmap)Image.FromStream(bmpstream);
				var bmp2 = (Bitmap)Image.FromStream(bmp2stream);

				var scn = GenieFile.Deserialize<Scenario>(srcstream);
				BitmapHelpers.FillMap(scn, bmp, bmp2);
				GenieFile.Serialize(scn, outstream);
			}
		}

		public Bmp2MapCommand()
		{
			MinArgumentCount = 4;
			Description = "Generate a scenario map based on bitmaps";
			Help = "Generate a scenario map based on bitmaps with automatic forest generation."
				+ "\n"
				+ "\nTwo bitmaps need to be specified: One terrain map and one elevation map."
				+ "\nBoth bitmaps need to be indexed or grayscale."
				+ "\nThe index/gray values of the first bitmap are translated directly to a terrain ID."
				+ "\nThe index/gray values of the second bitmap are divided by 37 to generate elevations 0-6."
				+ "\n"
				+ "\nParameters:"
				+ "\n<input scenario> <output scenario> <terrain bitmap> <elevation bitmap>"
			;
		}
	}
}

