﻿using System.IO;
using AdrianKousz.Util;

namespace AdrianKousz.GenieEngine
{
	public class ConvertCommand : CmdApp.Command
	{
		override public void Run(string[] args)
		{
			using (var srcstream = File.OpenRead(args[0]))
			using (var outstream = File.OpenWrite(args[1]))
			using (var refstream = File.OpenRead(args[2]))
			{
				var refscn = GenieFile.Deserialize<Scenario>(refstream);
				var srcscn = GenieFile.Deserialize<Scenario>(srcstream);

				srcscn.VersionString = refscn.VersionString;
				srcscn.VersionNumber = refscn.VersionNumber;
				srcscn.RawIndividualVictory = refscn.RawIndividualVictory;
				srcscn.RawDisables = refscn.RawDisables;
				srcscn.RawRemaining = refscn.RawRemaining;

				srcscn.PlayerSettings.ForEach(x => {
					x.FilenameAI = x.FilenameCTY = x.FilenamePER = "RandomGame";
					x.ScriptAI = x.ScriptCTY = x.ScriptPER = "";
				});

				var converter = new CopymapConverter();
				converter.Convert(srcscn);

				GenieFile.Serialize(srcscn, outstream);
			}
		}

		public ConvertCommand()
		{
			MinArgumentCount = 3;
			Description = "Convert AOE1 scenario";
			Help = "Convert AOE1 scenario to AOE2"
				+ "\n"
				+ "\nThe converter needs an existing AOE2 scenario to start with."
				+ "\n"
				+ "\nParameters:"
				+ "\n<AOE1 scenario> <output scenario> <AOE2 reference scenario>"
			;
		}
	}
}
