﻿using System;
using System.IO;
using AdrianKousz.Util;

namespace AdrianKousz.GenieEngine
{
	internal class CpnSerializerReader
	{
		private static readonly string TAG = typeof(CpnSerializerReader).Name;

		public static CpnSerializerReader CreateDefault()
		{
			return new CpnSerializerReader();
		}

		private CpnSerializerReader()
		{
		}

		public Campaign ReadCampaign(ExtendedBinaryReader reader)
		{
			var number = 0;
			var result = new Campaign();

			result.Version = reader.ReadString(4);
			if (!result.Version.Equals("1.00")) {
				Log.w(TAG, "Continue reading unsupported version {0}", result.Version);
			}

			result.Name = reader.ReadZString(256);
			number = reader.ReadInt32();

			result.Files = new PositionInfo()
				.CreateArray(number)
				.Fill(() => new PositionInfo {
						Size = reader.ReadInt32(),
						Offset = reader.ReadInt32(),
						Name = reader.ReadZString(255),
						Filename = reader.ReadZString(257),
					}
				)
				.Map(x => {
					var file = new Campaign.CpnFile();
					reader.BaseStream.Seek(x.Offset, SeekOrigin.Begin);
					file.RawData = reader.ReadBytes(x.Size);
					file.Name = x.Name;
					file.Filename = x.Filename;
					return file;
				})
				.ToArray();

			return result;
		}

		private class PositionInfo
		{
			public int Size;
			public int Offset;
			public string Name;
			public string Filename;
		}
	}
}
