﻿using System;
using AdrianKousz.Util;

namespace AdrianKousz.GenieEngine
{
	internal class CpnSerializerWriter
	{
		private static readonly string TAG = typeof(CpnSerializerWriter).Name;

		public static CpnSerializerWriter CreateDefault()
		{
			return new CpnSerializerWriter();
		}

		private CpnSerializerWriter()
		{
		}

		public void Write(ExtendedBinaryWriter writer, Campaign value)
		{
			var number = 0;

			if (!value.Version.Equals("1.00")) {
				Log.w(TAG, "Continue writing unsupported version {0}", value.Version);
			}

			var headerSize = 8 + 256;
			var listingSize = 8 + 255 + 257;
			var listingLength = listingSize * value.Files.Count;

			number = headerSize + listingLength;

			writer.WriteStringRaw(value.Version);
			writer.WriteZString(value.Name, 256);
			writer.Write((Int32)value.Files.Count);

			value.Files.ForEach(x => {
				writer.Write((Int32)x.RawData.Length);
				writer.Write((Int32)number);
				writer.WriteZString(x.Name, 255);
				writer.WriteZString(x.Filename, 257);
				number += x.RawData.Length;
			});

			value.Files.ForEach(x => {
				writer.Write(x.RawData);
			});
		}
	}
}
