﻿using System.Collections.Generic;
using AdrianKousz.Util;

namespace AdrianKousz.GenieEngine
{
	public class ScnDefaultFactory : IScnFactory
	{
		public const string FilenameGenerated = "GenieEditGenerated.scx";

		public Scenario MakeScenario()
		{
			return MakeScenario(true);
		}

		public Scenario MakeScenario(bool deep)
		{
			var result = new Scenario();
			result.Unknown1 = Scenario.ExpectedUnknown1;
			result.Unknown2 = Scenario.ExpectedUnknown2;
			result.Unknown3 = Scenario.ExpectedUnknown3;
			result.Unknown4 = Scenario.ExpectedUnknown4;
			result.Unknown5 = Scenario.ExpectedUnknown5;

			result.VersionString = "1.21";
			result.VersionNumber = 1.22f;
			result.Filename = FilenameGenerated;
			result.Timestamp = System.DateTime.Now;
			result.PlayerCount = 2;

			if (deep) {
				result.Messages = MakeMessages();
				result.Cinematics = MakeCinematics();
				result.GlobalVictory = MakeGlobalVictory();
				result.Map = MakeMap();
				result.PlayerSettings = result.PlayerSettings.NewList().Fill(Scenario.NumPlayers, MakePlayerSettings);
				result.ResourceCopies = result.ResourceCopies.NewList().Fill(Scenario.NumPlayerSections - 1, MakeResourceCopy);
				result.PlayerSettings2 = result.PlayerSettings2.NewList().Fill(Scenario.NumPlayerSections - 1, MakePlayerSettings2);
				result.Units = MakeUnitList(Scenario.NumPlayerSections);
			}

			return result;
		}

		public Scenario.ScnPlayerSettings MakePlayerSettings()
		{
			var result = new Scenario.ScnPlayerSettings();
			result.UnknownInfo = Scenario.ScnPlayerSettings.ExpectedUnknownInfo;
			result.UnknownResource = Scenario.ScnPlayerSettings.ExpectedUnknownResource;
			result.Name = "Player";
			result.NameStringID = -1;
			result.FilenameAI = result.FilenameCTY = result.FilenamePER = "RandomGame";
			result.ScriptAI = result.ScriptCTY = result.ScriptPER = "";
			result.AIType = 1;
			result.ResourceWood = 200;
			result.ResourceFood = 200;
			result.ResourceGold = 100;
			result.ResourceStone = 100;
			result.StartingAge = -1;
			result.Diplomacy = new int[Scenario.NumPlayers].Fill(3);
			return result;
		}

		public Scenario.ScnResourceCopy MakeResourceCopy()
		{
			// AOE2 standard
			var result = new Scenario.ScnResourceCopy();
			result.UnknownResource = Scenario.ScnResourceCopy.ExpectedUnknownResource;
			result.ResourceFood = 200;
			result.ResourceWood = 200;
			result.ResourceGold = 100;
			result.ResourceStone = 200;
			result.PopulationLimit = 75;
			return result;
		}

		public Scenario.ScnPlayerSettings2 MakePlayerSettings2()
		{
			var result = new Scenario.ScnPlayerSettings2();
			return result;
		}

		public Scenario.ScnMessages MakeMessages()
		{
			var result = new Scenario.ScnMessages();
			result.Instructions = "";
			result.Hints = "";
			result.Victory = "";
			result.Loss = "";
			result.History = "";
			result.Scouts = "";
			result.InstructionsStringID = -1;
			result.HintsStringID = -1;
			result.VictoryStringID = -1;
			result.LossStringID = -1;
			result.HistoryStringID = -1;
			result.ScoutsStringID = -1;
			return result;
		}

		public Scenario.ScnCinematics MakeCinematics()
		{
			var result = new Scenario.ScnCinematics();
			result.FilenamePregame = "";
			result.FilenameVictory = "";
			result.FilenameLoss = "";
			result.FilenameBitmap = "";
			result.BitmapUnknown = Scenario.ScnCinematics.ExpectedUnknownBitmapFalse;
			result.RawBitmap = new byte[0];
			return result;
		}

		public Scenario.ScnGlobalVictory MakeGlobalVictory()
		{
			var result = new Scenario.ScnGlobalVictory();
			result.RequireConquest = true;
			result.RequireAllCustom = false;
			return result;
		}

		public Scenario.ScnMap MakeMap()
		{
			var result = new Scenario.ScnMap();
			var size = 144;
			result.CameraX = -1;
			result.CameraY = -1;
			result.GeneratorId = 2;
			result.SizeX = size;
			result.SizeY = size;
			result.LinearTiles = new Scenario.ScnMap.Tile[size*size].Fill(() => new Scenario.ScnMap.Tile(0, 1, 0));
			return result;
		}

		public Scenario.ScnUnit MakeUnit()
		{
			var result = new Scenario.ScnUnit();
			result.Unknown1 = Scenario.ScnUnit.ExpectedUnknown1;
			result.Unknown2 = Scenario.ScnUnit.ExpectedUnknown2;
			result.GarrisonnedInId = -1;
			return result;
		}

		public static IList<IList<Scenario.ScnUnit>> MakeUnitList(int count = Scenario.NumPlayerSections)
		{
			IList<IList<Scenario.ScnUnit>> result = new List<IList<Scenario.ScnUnit>>()
				.Fill(count, () => (IList<Scenario.ScnUnit>)new List<Scenario.ScnUnit>());
			return result;
		}
	}
}

